@@ -51,6 +51,8 @@ static struct {
 	{ STATS_ARGS,         "bad compiler arguments         ", NULL, 0 },
 	{ STATS_NOTC,         "not a C/C++ file               ", NULL, 0 },
 	{ STATS_CONFTEST,     "autoconf compile/link          ", NULL, 0 },
+	{ STATS_UNSUPPORTED,  "unsupported compiler option    ", NULL, 0 },
+	{ STATS_OUTSTDOUT,    "output to stdout               ", NULL, 0 },
 	{ STATS_DEVICE,       "output to a non-regular file   ", NULL, 0 },
 	{ STATS_NOINPUT,      "no input file                  ", NULL, 0 },
 	{ STATS_NUMFILES,     "files in cache                 ", NULL, FLAG_NOZERO|FLAG_ALWAYS },
@@ -92,6 +94,13 @@ static void write_stats(int fd, unsigned counters[STATS_END])
 	write(fd, buf, len);
 }
 
+
+/* fill in some default stats values */
+static void stats_default(unsigned counters[STATS_END])
+{
+	counters[STATS_MAXSIZE] += DEFAULT_MAXSIZE / 16;
+}
+
 /* read in the stats from one dir and add to the counters */
 static void stats_read_fd(int fd, unsigned counters[STATS_END])
 {
@@ -99,6 +108,7 @@ static void stats_read_fd(int fd, unsigned counters[STATS_END])
 	int len;
 	len = read(fd, buf, sizeof(buf)-1);
 	if (len <= 0) {
+		stats_default(counters);
 		return;
 	}
 	buf[len] = 0;
@@ -183,7 +193,10 @@ void stats_read(const char *stats_file, unsigned counters[STATS_END])
 	int fd;
 
 	fd = open(stats_file, O_RDONLY);
-	if (fd == -1) return;
+	if (fd == -1) {
+		stats_default(counters);
+		return;
+	}
 	lock_fd(fd);
 	stats_read_fd(fd, counters);
 	close(fd);
@@ -209,6 +222,12 @@ void stats_summary(void)
 
 		stats_read(fname, counters);
 		free(fname);
+
+		/* oh what a nasty hack ... */
+		if (dir == -1) {
+			counters[STATS_MAXSIZE] = 0;
+		}
+
 	}
 
 	/* and display them */
@@ -315,7 +334,6 @@ void stats_set_sizes(const char *dir, size_t num_files, size_t total_size)
 	char *stats_file;
 
 	create_dir(dir);
-
 	x_asprintf(&stats_file, "%s/stats", dir);
 
 	memset(counters, 0, sizeof(counters));
